@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2006-2007, 2012-2014 Red Hat, Inc.
+# Copyright (C) 2006-2007, 2012-2015 Red Hat, Inc.
 # Copyright (C) 2006 Hugh O. Brock <hbrock@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,8 @@ import logging
 import traceback
 import collections
 
-# pylint: disable=E0611
 from gi.repository import Gtk
 from gi.repository import Gdk
-# pylint: enable=E0611
 
 import virtinst
 from virtinst import (VirtualChannelDevice, VirtualParallelDevice,
@@ -35,31 +33,33 @@ from virtinst import (VirtualChannelDevice, VirtualParallelDevice,
                       VirtualTPMDevice, VirtualPanicDevice)
 from virtinst import VirtualController
 
-from virtManager import uiutil
-from virtManager.fsdetails import vmmFSDetails
-from virtManager.gfxdetails import vmmGraphicsDetails
-from virtManager.netlist import vmmNetworkList
-from virtManager.asyncjob import vmmAsyncJob
-from virtManager.storagebrowse import vmmStorageBrowser
-from virtManager.baseclass import vmmGObjectUI
-from virtManager.addstorage import vmmAddStorage
+from . import uiutil
+from .fsdetails import vmmFSDetails
+from .gfxdetails import vmmGraphicsDetails
+from .netlist import vmmNetworkList
+from .asyncjob import vmmAsyncJob
+from .storagebrowse import vmmStorageBrowser
+from .baseclass import vmmGObjectUI
+from .addstorage import vmmAddStorage
 
-PAGE_ERROR = 0
-PAGE_DISK = 1
-PAGE_NETWORK = 2
-PAGE_INPUT = 3
-PAGE_GRAPHICS = 4
-PAGE_SOUND = 5
-PAGE_HOSTDEV = 6
-PAGE_CHAR = 7
-PAGE_VIDEO = 8
-PAGE_WATCHDOG = 9
-PAGE_FILESYSTEM = 10
-PAGE_SMARTCARD = 11
-PAGE_USBREDIR = 12
-PAGE_TPM = 13
-PAGE_RNG = 14
-PAGE_PANIC = 15
+(PAGE_ERROR,
+PAGE_DISK,
+PAGE_CONTROLLER,
+PAGE_NETWORK,
+PAGE_INPUT,
+PAGE_GRAPHICS,
+PAGE_SOUND,
+PAGE_HOSTDEV,
+PAGE_CHAR,
+PAGE_VIDEO,
+PAGE_WATCHDOG,
+PAGE_FILESYSTEM,
+PAGE_SMARTCARD,
+PAGE_USBREDIR,
+PAGE_TPM,
+PAGE_RNG,
+PAGE_PANIC,
+) = range(0, 17)
 
 
 class vmmAddHardware(vmmGObjectUI):
@@ -73,6 +73,8 @@ class vmmAddHardware(vmmGObjectUI):
         self.storage_browser = None
 
         self._dev = None
+        self._remove_usb_controller = None
+        self._selected_model = None
 
         self.gfxdetails = vmmGraphicsDetails(
             self.vm, self.builder, self.topwin)
@@ -97,7 +99,6 @@ class vmmAddHardware(vmmGObjectUI):
             "on_create_finish_clicked" : self.finish,
             "on_hw_list_changed": self.hw_selected,
 
-            "on_config_storage_bustype_changed": self.populate_disk_device,
             "on_config_storage_devtype_changed": self.change_storage_devtype,
 
             "on_mac_address_clicked" : self.change_macaddr_use,
@@ -113,6 +114,8 @@ class vmmAddHardware(vmmGObjectUI):
             "on_rng_type_changed": self.change_rng,
             "on_rng_backend_mode_changed": self.change_rng,
             "on_rng_backend_type_changed": self.change_rng,
+
+            "on_controller_type_changed": self.populate_controller_model,
         })
         self.bind_escape_key_close()
 
@@ -125,11 +128,12 @@ class vmmAddHardware(vmmGObjectUI):
         self.topwin.present()
         self.conn.schedule_priority_tick(pollnet=True,
                                          pollpool=True, polliface=True,
-                                         pollnodedev=True, pollmedia=True)
+                                         pollnodedev=True)
 
     def close(self, ignore1=None, ignore2=None):
-        logging.debug("Closing addhw")
-        self.topwin.hide()
+        if self.topwin.is_visible():
+            logging.debug("Closing addhw")
+            self.topwin.hide()
         if self.storage_browser:
             self.storage_browser.close()
 
@@ -195,11 +199,8 @@ class vmmAddHardware(vmmGObjectUI):
         self.build_network_model_combo(self.vm, netmodel_list)
 
         # Disk bus type
-        widget = self.widget("config-storage-bustype")
-        # [bus, label]
-        model = Gtk.ListStore(str, str)
-        widget.set_model(model)
-        uiutil.set_combo_text_column(widget, 1)
+        self.build_disk_bus_combo(self.vm,
+            self.widget("config-storage-bustype"))
 
         # Disk device type
         target_list = self.widget("config-storage-devtype")
@@ -214,6 +215,16 @@ class vmmAddHardware(vmmGObjectUI):
         text.set_property("xpad", 6)
         target_list.pack_start(text, True)
         target_list.add_attribute(text, 'text', 2)
+        target_model.append([virtinst.VirtualDisk.DEVICE_DISK,
+                      "drive-harddisk", _("Disk device")])
+        target_model.append([virtinst.VirtualDisk.DEVICE_CDROM,
+                      "media-cdrom", _("CDROM device")])
+        target_model.append([virtinst.VirtualDisk.DEVICE_FLOPPY,
+                      "media-floppy", _("Floppy device")])
+        if self.conn.is_qemu() or self.conn.is_test_conn():
+            target_model.append([virtinst.VirtualDisk.DEVICE_LUN,
+                          "drive-harddisk", _("LUN Passthrough")])
+        target_list.set_active(0)
 
         # Disk cache mode
         cache_list = self.widget("config-storage-cache")
@@ -226,7 +237,7 @@ class vmmAddHardware(vmmGObjectUI):
         input_list = self.widget("input-type")
         input_model = Gtk.ListStore(str, str, str)
         input_list.set_model(input_model)
-        uiutil.set_combo_text_column(input_list, 0)
+        uiutil.init_combo_text_column(input_list, 0)
 
         # Sound model list
         sound_list = self.widget("sound-model")
@@ -254,7 +265,7 @@ class vmmAddHardware(vmmGObjectUI):
         # Mode name, desc
         char_mode_model = Gtk.ListStore(str, str)
         char_mode.set_model(char_mode_model)
-        uiutil.set_combo_text_column(char_mode, 1)
+        uiutil.init_combo_text_column(char_mode, 1)
         char_mode_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         for t in VirtualSerialDevice.MODES:
             desc = VirtualSerialDevice.pretty_mode(t)
@@ -264,17 +275,17 @@ class vmmAddHardware(vmmGObjectUI):
         lst = self.widget("char-target-type")
         model = Gtk.ListStore(str, str)
         lst.set_model(model)
-        uiutil.set_combo_text_column(lst, 1)
+        uiutil.init_combo_text_column(lst, 1)
         if self.conn.is_qemu():
             model.append(["virtio", "virtio"])
         else:
-            model.append([None, "default"])
+            model.append([None, _("Hypervisor default")])
 
         # Char target name
         lst = self.widget("char-target-name")
         model = Gtk.ListStore(str)
         lst.set_model(model)
-        uiutil.set_combo_text_column(lst, 0)
+        uiutil.init_combo_text_column(lst, 0)
         for n in VirtualChannelDevice.CHANNEL_NAMES:
             model.append([n])
 
@@ -282,7 +293,7 @@ class vmmAddHardware(vmmGObjectUI):
         lst = self.widget("char-device-type")
         model = Gtk.ListStore(str, str)
         lst.set_model(model)
-        uiutil.set_combo_text_column(lst, 1)
+        uiutil.init_combo_text_column(lst, 1)
 
         # Watchdog widgets
         combo = self.widget("watchdog-model")
@@ -314,6 +325,16 @@ class vmmAddHardware(vmmGObjectUI):
         combo = self.widget("panic-type")
         self.build_panic_address_type(combo)
 
+        # Controller widgets
+        combo = self.widget("controller-type")
+        target_model = Gtk.ListStore(str, str)
+        combo.set_model(target_model)
+        uiutil.init_combo_text_column(combo, 1)
+        combo = self.widget("controller-model")
+        target_model = Gtk.ListStore(str, str)
+        combo.set_model(target_model)
+        uiutil.init_combo_text_column(combo, 1)
+
         # Available HW options
         is_local = not self.conn.is_remote()
         is_storage_capable = self.conn.is_storage_capable()
@@ -333,6 +354,7 @@ class vmmAddHardware(vmmGObjectUI):
 
         add_hw_option("Storage", "drive-harddisk", PAGE_DISK, have_storage,
                       have_storage and storage_tooltip or None)
+        add_hw_option("Controller", "device_pci", PAGE_CONTROLLER, True, None)
         add_hw_option("Network", "network-idle", PAGE_NETWORK, True, None)
         add_hw_option("Input", "input-mouse", PAGE_INPUT, self.vm.is_hvm(),
                       _("Not supported for this guest type."))
@@ -368,7 +390,7 @@ class vmmAddHardware(vmmGObjectUI):
         add_hw_option("Watchdog", "device_pci", PAGE_WATCHDOG,
                       self.vm.is_hvm(),
                       _("Not supported for this guest type."))
-        add_hw_option("Filesystem", Gtk.STOCK_DIRECTORY, PAGE_FILESYSTEM,
+        add_hw_option("Filesystem", "folder", PAGE_FILESYSTEM,
                       self.conn.check_support(
                         self.conn.SUPPORT_CONN_FILESYSTEM) and
                       not self.vm.stable_defaults(),
@@ -388,7 +410,8 @@ class vmmAddHardware(vmmGObjectUI):
     def reset_state(self):
         # Storage init
         self.populate_disk_format_combo_wrapper(True)
-        self.populate_disk_bus()
+        self.widget("config-storage-devtype").set_active(0)
+        self.widget("config-storage-devtype").emit("changed")
         self.addstorage.reset_state()
 
         # Network init
@@ -421,6 +444,7 @@ class vmmAddHardware(vmmGObjectUI):
         self.widget("char-target-type").set_active(0)
         self.widget("char-target-name").set_active(0)
         self.widget("char-path").set_text("")
+        self.widget("char-channel").set_text("")
         self.widget("char-host").set_text("127.0.0.1")
         self.widget("char-port").set_value(4555)
         self.widget("char-bind-host").set_text("127.0.0.1")
@@ -455,6 +479,9 @@ class vmmAddHardware(vmmGObjectUI):
         # Panic device params
         self.widget("panic-iobase").set_text("0x505")
 
+        # Controller device params
+        self.populate_controller_type()
+
         self.set_hw_selection(0)
 
 
@@ -489,7 +516,7 @@ class vmmAddHardware(vmmGObjectUI):
     def build_video_combo(vm, combo, no_default=None):
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         combo.get_model().set_sort_column_id(1, Gtk.SortType.ASCENDING)
 
         vmmAddHardware.populate_video_combo(vm, combo, no_default)
@@ -498,11 +525,11 @@ class vmmAddHardware(vmmGObjectUI):
     def build_sound_combo(vm, combo, no_default=False):
         model = Gtk.ListStore(str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 0)
+        uiutil.init_combo_text_column(combo, 0)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         stable_defaults = vm.stable_defaults()
-        stable_soundmodels = ["ich6", "ac97"]
+        stable_soundmodels = ["ich6", "ich9", "ac97"]
 
         for m in virtinst.VirtualAudio.MODELS:
             if m == virtinst.VirtualAudio.MODEL_DEFAULT and no_default:
@@ -520,7 +547,7 @@ class vmmAddHardware(vmmGObjectUI):
         ignore = vm
         model = Gtk.ListStore(str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 0)
+        uiutil.init_combo_text_column(combo, 0)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         for m in virtinst.VirtualWatchdog.MODELS:
@@ -535,7 +562,7 @@ class vmmAddHardware(vmmGObjectUI):
         ignore = vm
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         for m in virtinst.VirtualWatchdog.ACTIONS:
@@ -553,11 +580,12 @@ class vmmAddHardware(vmmGObjectUI):
         # [xml value, label]
         model.append([None, _("Hypervisor default")])
         if vm.is_hvm():
-            mod_list = ["rtl8139", "ne2k_pci", "pcnet", "e1000"]
+            mod_list = []
             if vm.get_hv_type() in ["kvm", "qemu", "test"]:
                 mod_list.append("virtio")
-            if (vm.get_hv_type() == "kvm" and
-                  vm.get_machtype() == "pseries"):
+            mod_list.append("rtl8139")
+            mod_list.append("e1000")
+            if vm.xmlobj.os.is_pseries():
                 mod_list.append("spapr-vlan")
             if vm.get_hv_type() in ["xen", "test"]:
                 mod_list.append("netfront")
@@ -570,7 +598,7 @@ class vmmAddHardware(vmmGObjectUI):
     def build_network_model_combo(vm, combo):
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         vmmAddHardware.populate_network_model_combo(vm, combo)
@@ -590,7 +618,7 @@ class vmmAddHardware(vmmGObjectUI):
     def build_smartcard_mode_combo(vm, combo):
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         vmmAddHardware.populate_smartcard_mode_combo(vm, combo)
@@ -618,7 +646,7 @@ class vmmAddHardware(vmmGObjectUI):
     def build_redir_type_combo(vm, combo):
         model = Gtk.ListStore(str, str, bool)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
 
         vmmAddHardware.populate_redir_type_combo(vm, combo)
         combo.set_active(0)
@@ -637,7 +665,7 @@ class vmmAddHardware(vmmGObjectUI):
     def build_tpm_type_combo(vm, combo):
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         vmmAddHardware.populate_tpm_type_combo(vm, combo)
@@ -656,13 +684,13 @@ class vmmAddHardware(vmmGObjectUI):
         ignore = vm
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
 
         combo.set_active(-1)
         for m in virtinst.VirtualDisk.cache_types:
             model.append([m, m])
 
-        _iter = model.insert(0, [None, "default"])
+        _iter = model.insert(0, [None, _("Hypervisor default")])
         combo.set_active_iter(_iter)
 
     @staticmethod
@@ -670,7 +698,7 @@ class vmmAddHardware(vmmGObjectUI):
         ignore = vm
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         combo.set_active(-1)
@@ -678,26 +706,60 @@ class vmmAddHardware(vmmGObjectUI):
             model.append([m, m])
 
         if not no_default:
-            model.append([None, "default"])
+            model.append([None, _("Hypervisor default")])
         combo.set_active(0)
 
     @staticmethod
-    def build_disk_bus_combo(vm, combo, no_default=False):
+    def build_disk_bus_combo(vm, combo):
         ignore = vm
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         model.set_sort_column_id(1, Gtk.SortType.ASCENDING)
-
-        if not no_default:
-            model.append([None, "default"])
         combo.set_active(-1)
 
+    @staticmethod
+    def populate_disk_bus_combo(vm, devtype, model, do_virtio_scsi=True):
+        rows = []
+        if vm.is_hvm():
+            if not vm.get_xmlobj().os.is_q35():
+                rows.append(["ide", "IDE"])
+            rows.append(["sata", "SATA"])
+            rows.append(["fdc", "Floppy"])
+
+            if not vm.stable_defaults():
+                rows.append(["scsi", "SCSI"])
+                rows.append(["usb", "USB"])
+
+        if vm.get_hv_type() in ["qemu", "kvm", "test"]:
+            rows.append(["sd", "SD"])
+            rows.append(["virtio", "VirtIO"])
+            if do_virtio_scsi:
+                rows.append(["virtio-scsi", "VirtIO SCSI"])
+            if vm.xmlobj.os.is_pseries():
+                rows.append(["spapr-vscsi", "sPAPR-vSCSI"])
+
+        if vm.conn.is_xen() or vm.conn.is_test_conn():
+            rows.append(["xen", "Xen"])
+
+        model.clear()
+
+        bus_map = {
+            "disk": ["ide", "sata", "scsi", "sd", "spapr-vscsi",
+                     "usb", "virtio", "virtio-scsi", "xen"],
+            "floppy": ["fdc"],
+            "cdrom": ["ide", "sata", "scsi"],
+            "lun": ["virtio-scsi"],
+        }
+        for row in rows:
+            if row[0] in bus_map[devtype]:
+                model.append(row)
+
     @staticmethod
     def populate_disk_format_combo(vm, combo, create):
         model = Gtk.ListStore(str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 0)
+        uiutil.init_combo_text_column(combo, 0)
 
         formats = ["raw", "qcow2", "qed"]
         no_create_formats = []
@@ -719,70 +781,68 @@ class vmmAddHardware(vmmGObjectUI):
                     combo.set_active_iter(row.iter)
                     break
 
+    @staticmethod
+    def populate_controller_model_combo(combo, controller_type, remove_widget,
+        add_default=False):
+        model = combo.get_model()
+        model.clear()
+
+        if remove_widget:
+            remove_widget.set_sensitive(
+                controller_type != virtinst.VirtualController.TYPE_USB)
+
+        if controller_type == virtinst.VirtualController.TYPE_USB:
+            model.append(["default", _("Hypervisor default")])
+            model.append(["ich9-ehci1", "USB 2"])
+            model.append(["nec-xhci", "USB 3"])
+        elif controller_type == virtinst.VirtualController.TYPE_SCSI:
+            model.append(["default", _("Hypervisor default")])
+            model.append(["virtio-scsi", "VirtIO SCSI"])
+        else:
+            if add_default:
+                model.append([None, _("Hypervisor default")])
+                uiutil.set_grid_row_visible(combo, False)
+
+    @staticmethod
+    def label_for_input_device(typ, bus):
+        if typ == "tablet" and bus == "usb":
+            return _("EvTouch USB Graphics Tablet")
+
+        if bus in ["usb", "ps2"]:
+            return _("Generic") + (" %s %s" %
+                (bus.upper(), str(typ).capitalize()))
+        return "%s %s" % (str(bus).capitalize(), str(typ).capitalize())
+
 
     #########################
     # UI population methods #
     #########################
 
-    def populate_disk_bus(self):
+    def refresh_disk_bus(self, devtype):
         widget = self.widget("config-storage-bustype")
         model = widget.get_model()
-        model.clear()
+        self.populate_disk_bus_combo(self.vm, devtype, model)
 
-        if self.vm.is_hvm():
-            model.append(["ide", "IDE"])
-            model.append(["fdc", "Floppy"])
+        # By default, select bus of the first disk assigned to the VM
+        default_bus = None
+        for i in self.vm.get_disk_devices():
+            if i.device == devtype:
+                default_bus = i.bus
+                break
 
-            if not self.vm.stable_defaults():
-                model.append(["scsi", "SCSI"])
-                model.append(["usb", "USB"])
-
-        if self.vm.get_hv_type() in ["qemu", "kvm", "test"]:
-            model.append(["sata", "SATA"])
-            model.append(["sd", "SD"])
-            model.append(["virtio", "VirtIO"])
-            model.append(["virtio-scsi", "VirtIO SCSI"])
-
-        if self.conn.is_xen() or self.conn.is_test_conn():
-            model.append(["xen", "Xen"])
-
-        if len(model) > 0:
+        if default_bus:
+            uiutil.select_list_row_by_value(widget, default_bus)
+        elif len(model) > 0:
             widget.set_active(0)
 
-    def populate_disk_device(self, src):
-        ignore = src
-
-        bus = self.get_config_disk_bus()
-        devlist = self.widget("config-storage-devtype")
-        model = devlist.get_model()
-        model.clear()
-
-        disk_buses = ["ide", "sata", "scsi", "sd",
-                      "usb", "virtio", "virtio-scsi", "xen"]
-        floppy_buses = ["fdc"]
-        cdrom_buses = ["ide", "scsi"]
-        lun_buses = ["virtio-scsi"]
-
-        if bus in disk_buses:
-            model.append([virtinst.VirtualDisk.DEVICE_DISK,
-                          "drive-harddisk", _("Disk device")])
-        if bus in floppy_buses:
-            model.append([virtinst.VirtualDisk.DEVICE_FLOPPY,
-                          "media-floppy", _("Floppy device")])
-        if bus in cdrom_buses:
-            model.append([virtinst.VirtualDisk.DEVICE_CDROM,
-                          "media-cdrom", _("CDROM device")])
-        if bus in lun_buses:
-            model.append([virtinst.VirtualDisk.DEVICE_LUN,
-                          "drive-harddisk", _("LUN device")])
-
-        if len(model) > 0:
-            devlist.set_active(0)
-
     def populate_input_model(self, model):
         model.clear()
-        model.append([_("EvTouch USB Graphics Tablet"), "tablet", "usb"])
-        model.append([_("Generic USB Mouse"), "mouse", "usb"])
+        def _add_row(typ, bus):
+            model.append([self.label_for_input_device(typ, bus), typ, bus])
+
+        _add_row("tablet", "usb")
+        _add_row("mouse", "usb")
+        _add_row("keyboard", "usb")
 
     def populate_host_device_model(self, devtype, devcap, subtype, subcap):
         devlist = self.widget("host-device")
@@ -791,21 +851,21 @@ class vmmAddHardware(vmmGObjectUI):
         subdevs = []
 
         if subtype:
-            subdevs = self.conn.get_nodedevs(subtype, subcap)
+            subdevs = self.conn.filter_nodedevs(subtype, subcap)
 
-        devs = self.conn.get_nodedevs(devtype, devcap)
+        devs = self.conn.filter_nodedevs(devtype, devcap)
         for dev in devs:
-            prettyname = dev.pretty_name()
+            prettyname = dev.xmlobj.pretty_name()
 
             for subdev in subdevs:
-                if dev.name == subdev.parent:
-                    prettyname += " (%s)" % subdev.pretty_name()
+                if dev.xmlobj.name == subdev.xmlobj.parent:
+                    prettyname += " (%s)" % subdev.xmlobj.pretty_name()
 
-            model.append([prettyname, dev.name, devtype, dev])
+            model.append([prettyname, dev.xmlobj.name, devtype, dev.xmlobj])
 
         if len(model) == 0:
             model.append([_("No Devices Available"), None, None, None])
-        uiutil.set_list_selection(devlist, 0)
+        uiutil.select_list_row_by_number(devlist, 0)
 
     def populate_disk_format_combo_wrapper(self, create):
         format_list = self.widget("config-storage-format")
@@ -813,6 +873,57 @@ class vmmAddHardware(vmmGObjectUI):
         if not create:
             format_list.get_child().set_text("")
 
+    def populate_controller_type(self):
+        widget = self.widget("controller-type")
+        model = widget.get_model()
+        model.clear()
+
+        for t in VirtualController.TYPES:
+            if t == VirtualController.TYPE_PCI:
+                continue
+            model.append([t, VirtualController.pretty_type(t)])
+
+        if len(model) > 0:
+            widget.set_active(0)
+
+    def populate_controller_model(self, src):
+        ignore = src
+
+        def show_tooltip(model_tooltip, show):
+            vmname = self.vm.get_name()
+            tooltip = (_("%s already has a USB controller attached.\n"
+            "Adding more than one USB controller is not supported.\n"
+            "You can change the USB controller type in the VM details screen.")
+            % vmname)
+            model_tooltip.set_visible(show)
+            model_tooltip.set_tooltip_text(tooltip)
+
+        controller_type = self.get_config_controller_type()
+        modellist = self.widget("controller-model")
+        modellist.set_sensitive(True)
+        model_tooltip = self.widget("controller-tooltip")
+        show_tooltip(model_tooltip, False)
+
+        controllers = self.vm.get_controller_devices()
+        if controller_type == VirtualController.TYPE_USB:
+            usb_controllers = [x for x in controllers if
+                    (x.type == VirtualController.TYPE_USB)]
+            if (len(usb_controllers) == 0):
+                self.widget("create-finish").set_sensitive(True)
+            elif (len(usb_controllers) == 1 and usb_controllers[0].model == "none"):
+                self._remove_usb_controller = usb_controllers[0]
+                self.widget("create-finish").set_sensitive(True)
+            else:
+                show_tooltip(model_tooltip, True)
+                self.widget("create-finish").set_sensitive(False)
+        else:
+            self.widget("create-finish").set_sensitive(True)
+        uiutil.set_grid_row_visible(modellist, True)
+        self.populate_controller_model_combo(modellist, controller_type, None, True)
+
+        if len(modellist.get_model()) > 0:
+            modellist.set_active(0)
+
 
     ########################
     # get_config_* methods #
@@ -821,7 +932,7 @@ class vmmAddHardware(vmmGObjectUI):
     def build_combo_with_values(self, combo, values, default=None):
         dev_model = Gtk.ListStore(str, str)
         combo.set_model(dev_model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
         dev_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
         types = combo.get_model()
@@ -905,12 +1016,12 @@ class vmmAddHardware(vmmGObjectUI):
 
     # Input getters
     def get_config_input(self):
-        row = uiutil.get_list_selection(self.widget("input-type"))
+        row = uiutil.get_list_selection(self.widget("input-type"), None)
         return row[1], row[2]
 
     # Network getters
     def get_config_net_model(self):
-        return uiutil.get_list_selection(self.widget("net-model"))[0]
+        return uiutil.get_combo_entry(self.widget("net-model"))
 
     def get_config_macaddr(self):
         macaddr = None
@@ -933,7 +1044,7 @@ class vmmAddHardware(vmmGObjectUI):
         return usb_info
 
     def get_config_host_device_info(self):
-        return uiutil.get_list_selection(self.widget("host-device"))
+        return uiutil.get_list_selection(self.widget("host-device"), None)
 
     # Video Getters
     def get_config_video_model(self):
@@ -1008,15 +1119,22 @@ class vmmAddHardware(vmmGObjectUI):
     def get_config_rng_backend_mode(self):
         return uiutil.get_list_selection(self.widget("rng-backend-mode"), 0)
 
+    # CONTROLLER getters
+    def get_config_controller_type(self):
+        return uiutil.get_list_selection(self.widget("controller-type"), 0)
+
+    def get_config_controller_model(self):
+        return uiutil.get_list_selection(self.widget("controller-model"), 0)
+
     ################
     # UI listeners #
     ################
 
     def set_hw_selection(self, page):
-        uiutil.set_list_selection(self.widget("hw-list"), page)
+        uiutil.select_list_row_by_number(self.widget("hw-list"), page)
 
     def get_hw_selection(self):
-        return uiutil.get_list_selection(self.widget("hw-list"))
+        return uiutil.get_list_selection(self.widget("hw-list"), None)
 
     def update_char_device_type_model(self):
         stable_blacklist = ["pipe", "udp"]
@@ -1052,6 +1170,8 @@ class vmmAddHardware(vmmGObjectUI):
         sens = row[3]
         msg = row[4] or ""
 
+        self.widget("create-finish").set_sensitive(sens)
+
         if not sens:
             page = PAGE_ERROR
             self.widget("hardware-info").set_text(msg)
@@ -1079,6 +1199,8 @@ class vmmAddHardware(vmmGObjectUI):
 
     def change_storage_devtype(self, ignore):
         devtype = self.get_config_disk_device()
+        self.refresh_disk_bus(devtype)
+
         allow_create = devtype not in ["cdrom", "floppy"]
         self.addstorage.widget("config-storage-create-box").set_sensitive(
             allow_create)
@@ -1113,6 +1235,8 @@ class vmmAddHardware(vmmGObjectUI):
             return _("Error")
         if page == PAGE_DISK:
             return _("Storage")
+        if page == PAGE_CONTROLLER:
+            return _("Controller")
         if page == PAGE_NETWORK:
             return _("Network")
         if page == PAGE_INPUT:
@@ -1184,10 +1308,13 @@ class vmmAddHardware(vmmGObjectUI):
         settype = None
         if text == VirtualChannelDevice.CHANNEL_NAME_SPICE:
             settype = "spicevmc"
+        elif text == VirtualChannelDevice.CHANNEL_NAME_SPICE_WEBDAV:
+            settype = "spiceport"
+            self.widget("char-channel").set_text(text)
         elif (text == VirtualChannelDevice.CHANNEL_NAME_QEMUGA or
               text == VirtualChannelDevice.CHANNEL_NAME_LIBGUESTFS):
             settype = "unix"
-        uiutil.set_row_selection(self.widget("char-device-type"), settype)
+        uiutil.select_list_row_by_value(self.widget("char-device-type"), settype)
 
     def change_char_device_type(self, src):
         devtype = uiutil.get_list_selection(src, 0)
@@ -1196,6 +1323,7 @@ class vmmAddHardware(vmmGObjectUI):
 
         char_widget_mappings = {
             "source_path" : "char-path",
+            "source_channel" : "char-channel",
             "source_mode" : "char-mode",
             "source_host" : "char-host",
             "bind_host" : "char-bind-host",
@@ -1263,15 +1391,103 @@ class vmmAddHardware(vmmGObjectUI):
     # Add device methods #
     ######################
 
+    @staticmethod
+    def change_config_helper(define_func, define_args, vm, err,
+            devobj=None, hotplug_args=None):
+        hotplug_args = hotplug_args or {}
+
+        # Persistent config change
+        try:
+            if devobj:
+                # Device XML editing
+                define_func(devobj=devobj, do_hotplug=False, **define_args)
+            else:
+                # Guest XML editing
+                define_func(**define_args)
+        except Exception, e:
+            err.show_err((_("Error changing VM configuration: %s") %
+                              str(e)))
+            return False
+
+        if not vm.is_active():
+            return True
+
+        # Hotplug change
+        hotplug_err = None
+        did_hotplug = False
+        try:
+            if devobj:
+                define_func(devobj=devobj, do_hotplug=True, **define_args)
+                did_hotplug = True
+            elif hotplug_args:
+                did_hotplug = True
+                vm.hotplug(**hotplug_args)
+        except Exception, e:
+            did_hotplug = True
+            logging.debug("Hotplug failed: %s", str(e))
+            hotplug_err = ((str(e), "".join(traceback.format_exc())))
+
+        if did_hotplug and not hotplug_err:
+            return True
+
+        if len(define_args) > 1:
+            msg = _("Some changes may require a guest shutdown "
+                    "to take effect.")
+        else:
+            msg = _("These changes will take effect after "
+                    "the next guest shutdown.")
+
+        dtype = (hotplug_err and
+                 Gtk.MessageType.WARNING or Gtk.MessageType.INFO)
+        hotplug_msg = ""
+        if hotplug_err:
+            hotplug_msg += (hotplug_err[0] + "\n\n" +
+                            hotplug_err[1] + "\n")
+
+        err.show_err(msg,
+                details=hotplug_msg,
+                buttons=Gtk.ButtonsType.OK,
+                dialog_type=dtype)
+
+        return True
+
     def setup_device(self, asyncjob):
-        logging.debug("Running setup for device=%s", self._dev)
+        poolname = None
+        if (self._dev.virtual_device_type == "disk" and
+            self._dev.wants_storage_creation() and
+            self._dev.get_parent_pool()):
+            poolname = self._dev.get_parent_pool().name()
+
+        logging.debug("Running setup() for device=%s", self._dev)
         self._dev.setup(meter=asyncjob.get_meter())
-        logging.debug("Setup complete")
+        logging.debug("Device setup() complete")
+
+        if poolname:
+            try:
+                pool = self.conn.get_pool(poolname)
+                self.idle_add(pool.refresh)
+            except:
+                logging.debug("Error looking up pool=%s for refresh after "
+                    "storage creation.", poolname, exc_info=True)
+
 
     def add_device(self):
         self._dev.get_xml_config()
         logging.debug("Adding device:\n" + self._dev.get_xml_config())
 
+        if self._remove_usb_controller:
+            kwargs = {}
+            kwargs["model"] = self._selected_model
+
+            self.change_config_helper(self.vm.define_controller,
+                    kwargs, self.vm, self.err,
+                    devobj=self._remove_usb_controller)
+
+            self._remove_usb_controller = None
+            self._selected_model = None
+
+            return
+
         controller = getattr(self._dev, "vmm_controller", None)
         if controller is not None:
             logging.debug("Adding controller:\n%s",
@@ -1366,6 +1582,8 @@ class vmmAddHardware(vmmGObjectUI):
             return True
         elif page_num == PAGE_DISK:
             return self.validate_page_storage()
+        elif page_num == PAGE_CONTROLLER:
+            return self.validate_page_controller()
         elif page_num == PAGE_NETWORK:
             return self.validate_page_network()
         elif page_num == PAGE_INPUT:
@@ -1466,20 +1684,16 @@ class vmmAddHardware(vmmGObjectUI):
                 disk.driver_cache = cache
 
             # Generate target
-            disks = []
-            if not self.is_customize_dialog:
-                disks = (self.vm.get_disk_devices() +
-                         self.vm.get_disk_devices(inactive=True))
-                for d in disks:
-                    if d.target not in used:
-                        used.append(d.target)
+            disks = (self.vm.get_disk_devices() +
+                     self.vm.get_disk_devices(inactive=True))
+            for d in disks:
+                if d.target not in used:
+                    used.append(d.target)
 
             prefer_ctrl = self._set_disk_controller(
                 disk, controller_model, disks)
 
-            if not self.is_customize_dialog:
-                disk.generate_target(used, prefer_ctrl)
-
+            disk.generate_target(used, prefer_ctrl)
         except Exception, e:
             return self.err.val_err(_("Storage parameter error."), e)
 
@@ -1554,7 +1768,7 @@ class vmmAddHardware(vmmGObjectUI):
         if devtype == "usb_device":
             vendor = nodedev.vendor_id
             product = nodedev.product_id
-            count = self.conn.get_nodedevs_number(devtype, vendor, product)
+            count = self.conn.get_nodedev_count(devtype, vendor, product)
             if not count:
                 raise RuntimeError(_("Could not find USB device "
                                      "(vendorId: %s, productId: %s) "
@@ -1564,6 +1778,19 @@ class vmmAddHardware(vmmGObjectUI):
 
         try:
             dev = virtinst.VirtualHostDevice(self.conn.get_backend())
+            # Hostdev collision
+            names  = []
+            for vm in self.conn.list_vms():
+                for hostdev in vm.get_hostdev_devices():
+                    if nodedev.compare_to_hostdev(hostdev):
+                        names.append(vm.get_name())
+            if names:
+                res = self.err.yes_no(
+                        _('The device is already in use by other guests %s') %
+                         (names),
+                        _("Do you really want to use the device?"))
+                if not res:
+                    return False
             dev.set_from_nodedev(nodedev, use_full_usb=is_dup)
             self._dev = dev
         except Exception, e:
@@ -1581,6 +1808,7 @@ class vmmAddHardware(vmmGObjectUI):
         devclass.type = devtype
 
         source_path = self.widget("char-path").get_text()
+        source_channel = self.widget("char-channel").get_text()
         source_mode = uiutil.get_list_selection(modebox, 0)
         source_host = self.widget("char-host").get_text()
         bind_host = self.widget("char-bind-host").get_text()
@@ -1605,6 +1833,7 @@ class vmmAddHardware(vmmGObjectUI):
 
         value_mappings = {
             "source_path" : source_path,
+            "source_channel" : source_channel,
             "source_mode" : source_mode,
             "source_host" : source_host,
             "source_port" : source_port,
@@ -1719,6 +1948,27 @@ class vmmAddHardware(vmmGObjectUI):
         except Exception, e:
             return self.err.val_err(_("Panic device parameter error"), e)
 
+    def validate_page_controller(self):
+        conn = self.conn.get_backend()
+        controller_type = self.get_config_controller_type()
+        model = self.get_config_controller_model()
+        self._dev = VirtualController(conn)
+        self._selected_model = model
+
+        controllers = self.vm.get_controller_devices()
+        controller_num = [x for x in controllers if
+                (x.type == controller_type)]
+        if len(controller_num) > 0:
+            index_new = max([x.index for x in controller_num]) + 1
+            self._dev.index = index_new
+
+        self._dev.type = controller_type
+
+        if model != "none":
+            if model == "default":
+                model = None
+            self._dev.model = model
+
     def validate_page_rng(self):
         conn = virtinst.VirtualRNGDevice.BACKEND_MODE_CONNECT in \
                self.get_config_rng_backend_mode()
@@ -1793,9 +2043,8 @@ class vmmAddHardware(vmmGObjectUI):
         if self.storage_browser is None:
             self.storage_browser = vmmStorageBrowser(conn)
 
-        self.storage_browser.stable_defaults = self.vm.stable_defaults()
-
+        self.storage_browser.set_stable_defaults(self.vm.stable_defaults())
         self.storage_browser.set_finish_cb(set_storage_cb)
         self.storage_browser.set_browse_reason(reason)
 
-        self.storage_browser.show(self.topwin, conn)
+        self.storage_browser.show(self.topwin)
