@@ -25,9 +25,9 @@ import logging
 
 from virtinst import Interface, InterfaceProtocol
 
-from virtManager import uiutil
-from virtManager.baseclass import vmmGObjectUI
-from virtManager.asyncjob import vmmAsyncJob
+from . import uiutil
+from .baseclass import vmmGObjectUI
+from .asyncjob import vmmAsyncJob
 
 PAGE_TYPE = 0
 PAGE_DETAILS = 1
@@ -178,7 +178,7 @@ class vmmCreateInterface(vmmGObjectUI):
     def build_interface_startmode_combo(combo):
         model = Gtk.ListStore(str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 0)
+        uiutil.init_combo_text_column(combo, 0)
 
         model.append(["none"])
         model.append(["onboot"])
@@ -195,7 +195,7 @@ class vmmCreateInterface(vmmGObjectUI):
         type_list = self.widget("interface-type")
         type_model = Gtk.ListStore(str, str)
         type_list.set_model(type_model)
-        uiutil.set_combo_text_column(type_list, 1)
+        uiutil.init_combo_text_column(type_list, 1)
         type_model.append([Interface.INTERFACE_TYPE_BRIDGE,
                            _("Bridge")])
         type_model.append([Interface.INTERFACE_TYPE_BOND,
@@ -255,7 +255,7 @@ class vmmCreateInterface(vmmGObjectUI):
         mode_list = self.widget("bond-mode")
         mode_model = Gtk.ListStore(str, str)
         mode_list.set_model(mode_model)
-        uiutil.set_combo_text_column(mode_list, 0)
+        uiutil.init_combo_text_column(mode_list, 0)
         mode_model.append([_("System default"), None])
         for m in Interface.INTERFACE_BOND_MODES:
             mode_model.append([m, m])
@@ -263,7 +263,7 @@ class vmmCreateInterface(vmmGObjectUI):
         mon_list = self.widget("bond-monitor-mode")
         mon_model = Gtk.ListStore(str, str)
         mon_list.set_model(mon_model)
-        uiutil.set_combo_text_column(mon_list, 0)
+        uiutil.init_combo_text_column(mon_list, 0)
         mon_model.append([_("System default"), None])
         for m in Interface.INTERFACE_BOND_MONITOR_MODES:
             mon_model.append([m, m])
@@ -271,14 +271,14 @@ class vmmCreateInterface(vmmGObjectUI):
         validate_list = self.widget("arp-validate")
         validate_model = Gtk.ListStore(str)
         validate_list.set_model(validate_model)
-        uiutil.set_combo_text_column(validate_list, 0)
+        uiutil.init_combo_text_column(validate_list, 0)
         for m in Interface.INTERFACE_BOND_MONITOR_MODE_ARP_VALIDATE_MODES:
             validate_model.append([m])
 
         carrier_list = self.widget("mii-carrier")
         carrier_model = Gtk.ListStore(str)
         carrier_list.set_model(carrier_model)
-        uiutil.set_combo_text_column(carrier_list, 0)
+        uiutil.init_combo_text_column(carrier_list, 0)
         for m in Interface.INTERFACE_BOND_MONITOR_MODE_MII_CARRIER_TYPES:
             carrier_model.append([m])
 
@@ -286,13 +286,13 @@ class vmmCreateInterface(vmmGObjectUI):
         copy_iface = self.widget("ip-copy-interface-combo")
         copy_model = Gtk.ListStore(str, object, bool)
         copy_iface.set_model(copy_model)
-        uiutil.set_combo_text_column(copy_iface, 0)
+        uiutil.init_combo_text_column(copy_iface, 0)
         copy_iface.add_attribute(txt, "sensitive", 2)
 
         ip_mode = self.widget("ipv4-mode")
         ip_model = Gtk.ListStore(str)
         ip_mode.set_model(ip_model)
-        uiutil.set_combo_text_column(ip_mode, 0)
+        uiutil.init_combo_text_column(ip_mode, 0)
         ip_model.insert(IP_DHCP, ["DHCP"])
         ip_model.insert(IP_STATIC, ["Static"])
         ip_model.insert(IP_NONE, ["No configuration"])
@@ -300,7 +300,7 @@ class vmmCreateInterface(vmmGObjectUI):
         ip_mode = self.widget("ipv6-mode")
         ip_model = Gtk.ListStore(str)
         ip_mode.set_model(ip_model)
-        uiutil.set_combo_text_column(ip_mode, 0)
+        uiutil.init_combo_text_column(ip_mode, 0)
         ip_model.insert(IP_DHCP, ["DHCP"])
         ip_model.insert(IP_STATIC, ["Static"])
         ip_model.insert(IP_NONE, ["No configuration"])
@@ -501,11 +501,11 @@ class vmmCreateInterface(vmmGObjectUI):
         self.widget("interface-list-text").set_text(msg)
 
         nodedevs = {}
-        for phys in self.conn.get_nodedevs("net"):
-            nodedevs[phys.interface] = [None,
-                                        False, False, phys.interface,
-                                        "ethernet", False, True, None,
-                                        phys.address]
+        for phys in self.conn.filter_nodedevs("net"):
+            nodedevs[phys.xmlobj.interface] = [None,
+                False, False, phys.xmlobj.interface,
+                "ethernet", False, True, None,
+                phys.xmlobj.address]
 
         row_dict = {}
         for iface in self.conn.list_interfaces():
@@ -789,9 +789,14 @@ class vmmCreateInterface(vmmGObjectUI):
         ipv6 = None
 
         if not is_manual:
-            if copy_row[1] and copy_row[2]:
-                copy_name = copy_row[1].get_name()
-                proto_xml = copy_row[1].get_protocol_xml()
+            copy_vmmiface = copy_row[1]
+            copy_cancopy = copy_row[2]
+            if copy_vmmiface and copy_cancopy:
+                copy_name = copy_vmmiface.get_name()
+                # We always want the inactive protocol XML, which
+                # will list the on disk config, not the run time config,
+                # which doesn't list DHCP
+                proto_xml = copy_vmmiface.get_protocol_xml(inactive=True)
 
         else:
             # Build IPv4 Info
@@ -926,8 +931,19 @@ class vmmCreateInterface(vmmGObjectUI):
             if (itype == Interface.INTERFACE_TYPE_BRIDGE or
                 itype == Interface.INTERFACE_TYPE_BOND):
                 for row in ifaces:
-                    child = Interface(self.conn.get_backend(),
-                        parsexml=row[INTERFACE_ROW_KEY].get_xml_config())
+                    if row[INTERFACE_ROW_IS_DEFINED]:
+                        vmmiface = self.conn.get_interface(
+                            row[INTERFACE_ROW_NAME])
+
+                        # Use the inactive XML, which drops a bunch
+                        # elements that might cause netcf to choke on
+                        # for a sub-interface
+                        xml = vmmiface.get_xmlobj(
+                            inactive=True).get_xml_config()
+                    else:
+                        xml = row[INTERFACE_ROW_KEY].get_xml_config()
+
+                    child = Interface(self.conn.get_backend(), parsexml=xml)
                     iobj.add_interface(child)
                 check_conflict = True
 
