@@ -22,7 +22,7 @@ from gi.repository import Gtk
 import logging
 import traceback
 
-from virtManager.baseclass import vmmGObject
+from .baseclass import vmmGObject
 
 
 def _launch_dialog(dialog, primary_text, secondary_text, title,
@@ -54,19 +54,36 @@ class vmmErrorDialog(vmmGObject):
         self._parent = parent
         self._simple = None
 
-    def _cleanup(self):
-        pass
+        # Callback to lookup the parent window if none is specified.
+        # Used by engine.py for properly parenting windows
+        self._find_parent_cb = None
 
+        # Allows the error owner to easily override default modality
+        self._modal_default = False
+
+    def _cleanup(self):
+        self._find_parent_cb = None
+
+    def set_modal_default(self, val):
+        self._modal_default = val
+    def set_find_parent_cb(self, cb):
+        self._find_parent_cb = cb
     def set_parent(self, parent):
         self._parent = parent
     def get_parent(self):
-        return self._parent
+        parent = self._parent
+        if parent is None and self._find_parent_cb:
+            parent = self._find_parent_cb()
+        return parent
 
     def show_err(self, summary, details=None, title="",
-                 modal=False, debug=True,
+                 modal=None, debug=True,
                  dialog_type=Gtk.MessageType.ERROR,
                  buttons=Gtk.ButtonsType.CLOSE,
                  text2=None):
+        if modal is None:
+            modal = self._modal_default
+
         if details is None:
             details = summary
             tb = "".join(traceback.format_exc()).strip()
@@ -93,6 +110,7 @@ class vmmErrorDialog(vmmGObject):
                                   details=details, title=title,
                                   modal=modal)
 
+
     ###################################
     # Simple one shot message dialogs #
     ###################################
@@ -215,7 +233,7 @@ class vmmErrorDialog(vmmGObject):
         @dialog_type: Maps to FileChooserDialog 'action'
         @confirm_func: Optional callback function if file is chosen.
         @browse_reason: The vmmConfig.CONFIG_DIR* reason we are browsing.
-            If set, this will override the 'folder' parameter with the gconf
+            If set, this will override the 'folder' parameter with the gsettings
             value, and store the user chosen path.
         """
         import os
@@ -234,7 +252,7 @@ class vmmErrorDialog(vmmGObject):
             choose_button = Gtk.STOCK_OPEN
 
         fcdialog = Gtk.FileChooserDialog(title=dialog_name,
-                                    parent=self._parent,
+                                    parent=self.get_parent(),
                                     action=dialog_type,
                                     buttons=(Gtk.STOCK_CANCEL,
                                              Gtk.ResponseType.CANCEL,
@@ -280,7 +298,7 @@ class vmmErrorDialog(vmmGObject):
             ret = fcdialog.get_filename()
         fcdialog.destroy()
 
-        # Store the chosen directory in gconf if necessary
+        # Store the chosen directory in gsettings if necessary
         if ret and browse_reason and not ret.startswith("/dev"):
             self.config.set_default_directory(
                 os.path.dirname(ret), browse_reason)
