@@ -27,11 +27,10 @@ from gi.repository import GdkPixbuf
 
 from virtinst import util
 
-from virtManager import vmmenu
-from virtManager import uiutil
-from virtManager.connection import vmmConnection
-from virtManager.baseclass import vmmGObjectUI
-from virtManager.graphwidgets import CellRendererSparkline
+from . import vmmenu
+from . import uiutil
+from .baseclass import vmmGObjectUI
+from .graphwidgets import CellRendererSparkline
 
 import libvirt
 
@@ -251,34 +250,34 @@ class vmmManager(vmmGObjectUI):
     ################
 
     def init_stats(self):
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_guest_cpu_usage_visible_changed(
                                 self.toggle_guest_cpu_usage_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_host_cpu_usage_visible_changed(
                                 self.toggle_host_cpu_usage_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_memory_usage_visible_changed(
                                 self.toggle_memory_usage_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_disk_io_visible_changed(
                                 self.toggle_disk_io_visible_widget))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_vmlist_network_traffic_visible_changed(
                                 self.toggle_network_traffic_visible_widget))
 
         # Register callbacks with the global stats enable/disable values
         # that disable the associated vmlist widgets if reporting is disabled
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_cpu_poll_changed(
                 self.enable_polling, COL_GUEST_CPU))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_disk_poll_changed(
                 self.enable_polling, COL_DISK))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_net_poll_changed(
                 self.enable_polling, COL_NETWORK))
-        self.add_gconf_handle(
+        self.add_gsettings_handle(
             self.config.on_stats_enable_memory_poll_changed(
                 self.enable_polling, COL_MEM))
 
@@ -566,12 +565,12 @@ class vmmManager(vmmGObjectUI):
 
     def close_conn(self, ignore):
         conn = self.current_conn()
-        if conn.get_state() != vmmConnection.STATE_DISCONNECTED:
+        if not conn.is_disconnected():
             conn.close()
 
     def open_conn(self, ignore=None):
         conn = self.current_conn()
-        if conn.get_state() == vmmConnection.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             conn.open()
             return True
 
@@ -588,8 +587,7 @@ class vmmManager(vmmGObjectUI):
         if self.vm_row_key(vm) in self.rows:
             return
 
-        vm.connect("config-changed", self.vm_config_changed)
-        vm.connect("status-changed", self.vm_status_changed)
+        vm.connect("state-changed", self.vm_changed)
         vm.connect("resources-sampled", self.vm_row_updated)
         vm.connect("inspection-changed", self.vm_inspection_changed)
 
@@ -612,16 +610,16 @@ class vmmManager(vmmGObjectUI):
 
     def _build_conn_hint(self, conn):
         hint = conn.get_uri()
-        if conn.state == conn.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             hint += " (%s)" % _("Double click to connect")
         return hint
 
     def _build_conn_markup(self, conn, name):
         name = util.xml_escape(name)
         text = name
-        if conn.state == conn.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             text += " - " + _("Not Connected")
-        elif conn.state == conn.STATE_CONNECTING:
+        elif conn.is_connecting():
             text += " - " + _("Connecting...")
 
         markup = "<span size='smaller'>%s</span>" % text
@@ -629,7 +627,7 @@ class vmmManager(vmmGObjectUI):
 
     def _build_conn_color(self, conn):
         color = "#000000"
-        if conn.state == conn.STATE_DISCONNECTED:
+        if conn.is_disconnected():
             color = "#5b5b5b"
         return color
 
@@ -641,7 +639,7 @@ class vmmManager(vmmGObjectUI):
 
     def _build_row(self, conn, vm):
         if conn:
-            name = conn.get_pretty_desc(shorthost=False)
+            name = conn.get_pretty_desc()
             markup = self._build_conn_markup(conn, name)
             status = ("<span size='smaller'>%s</span>" %
                       conn.get_state_text())
@@ -666,7 +664,7 @@ class vmmManager(vmmGObjectUI):
         row.insert(ROW_HINT, util.xml_escape(hint))
         row.insert(ROW_IS_CONN, bool(conn))
         row.insert(ROW_IS_CONN_CONNECTED,
-                   bool(conn) and conn.state != conn.STATE_DISCONNECTED)
+                   bool(conn) and not conn.is_disconnected())
         row.insert(ROW_IS_VM, bool(vm))
         row.insert(ROW_IS_VM_RUNNING, bool(vm) and vm.is_active())
         row.insert(ROW_COLOR, color)
@@ -697,40 +695,6 @@ class vmmManager(vmmGObjectUI):
         self.rows[conn.get_uri()] = model[path]
         return _iter
 
-    def _ensure_conn_descs_dont_collide(self):
-        # By default we only show hostname + hypervisor in the conn label.
-        # So if we have two URIs like qemu+ssh://host and qemu+tcp://host,
-        # we want to add the transport in the description to differentiate
-        connrows = [row for row in self.rows.values() if row[ROW_IS_CONN]]
-        for row in connrows:
-            conn = row[ROW_HANDLE]
-            connsplit = util.uri_split(conn.get_uri())
-            scheme = connsplit[0]
-
-            show_transport = False
-            show_user = False
-
-            for checkrow in connrows:
-                checkconn = checkrow[ROW_HANDLE]
-                if conn is checkconn:
-                    continue
-                checkconnsplit = util.uri_split(checkconn.get_uri())
-                checkscheme = checkconnsplit[0]
-
-                if ((scheme.split("+")[0] == checkscheme.split("+")[0]) and
-                    connsplit[2] == checkconnsplit[2] and
-                    connsplit[3] == checkconnsplit[3]):
-                    show_transport = True
-                    if ("+" in scheme and "+" in checkscheme and
-                        scheme.split("+")[1] == checkscheme.split("+")[1]):
-                        show_user = True
-
-            newname = conn.get_pretty_desc(
-                shorthost=False, show_transport=show_transport,
-                show_user=show_user)
-            if newname != row[ROW_SORT_KEY]:
-                self.conn_state_changed(conn, newname=newname)
-
     def add_conn(self, engine_ignore, conn):
         # Called from engine.py signal conn-added
 
@@ -747,7 +711,6 @@ class vmmManager(vmmGObjectUI):
 
         vmlist = self.widget("vm-list")
         self._append_conn(vmlist.get_model(), conn)
-        self._ensure_conn_descs_dont_collide()
 
     def remove_conn(self, engine_ignore, uri):
         # Called from engine.py signal conn-removed
@@ -778,12 +741,15 @@ class vmmManager(vmmGObjectUI):
             return
         self.widget("vm-list").get_model().row_changed(row.path, row.iter)
 
-    def vm_config_changed(self, vm):
+    def vm_changed(self, vm):
         row = self.rows.get(self.vm_row_key(vm), None)
         if row is None:
             return
 
         try:
+            if vm == self.current_vm():
+                self.update_current_selection()
+
             name = vm.get_name_or_title()
             status = vm.run_status()
 
@@ -803,27 +769,6 @@ class vmmManager(vmmGObjectUI):
 
         self.vm_row_updated(vm)
 
-    def vm_status_changed(self, vm, oldstatus, newstatus):
-        ignore = newstatus
-        ignore = oldstatus
-        parent = self.rows[vm.conn.get_uri()].iter
-        vmlist = self.widget("vm-list")
-        model = vmlist.get_model()
-
-        missing = True
-        for row in range(model.iter_n_children(parent)):
-            _iter = model.iter_nth_child(parent, row)
-            if model[_iter][ROW_HANDLE] == vm:
-                missing = False
-                break
-
-        if missing:
-            self._append_vm(model, vm, vm.conn)
-
-        # Update run/shutdown/pause button states
-        self.update_current_selection()
-        self.vm_config_changed(vm)
-
     def vm_inspection_changed(self, vm):
         row = self.rows.get(self.vm_row_key(vm), None)
         if row is None:
@@ -859,17 +804,15 @@ class vmmManager(vmmGObjectUI):
 
             it = model.iter_next(it)
 
-    def conn_state_changed(self, conn, newname=None):
+    def conn_state_changed(self, conn):
         row = self.rows[conn.get_uri()]
-        if newname:
-            row[ROW_SORT_KEY] = newname
+        row[ROW_SORT_KEY] = conn.get_pretty_desc()
         row[ROW_MARKUP] = self._build_conn_markup(conn, row[ROW_SORT_KEY])
-        row[ROW_IS_CONN_CONNECTED] = conn.state != conn.STATE_DISCONNECTED
+        row[ROW_IS_CONN_CONNECTED] = not conn.is_disconnected()
         row[ROW_COLOR] = self._build_conn_color(conn)
         row[ROW_HINT] = self._build_conn_hint(conn)
 
-        if conn.get_state() in [vmmConnection.STATE_DISCONNECTED,
-                                vmmConnection.STATE_CONNECTING]:
+        if not conn.is_active():
             # Connection went inactive, delete any VM child nodes
             parent = row.iter
             if parent is not None:
@@ -919,7 +862,7 @@ class vmmManager(vmmGObjectUI):
         show_shutdown = bool(vm and vm.is_stoppable())
 
         if vm and vm.managedsave_supported:
-            self.change_run_text(vm.hasSavedImage())
+            self.change_run_text(vm.has_managed_save())
 
         self.widget("vm-open").set_sensitive(show_open)
         self.widget("vm-run").set_sensitive(show_run)
@@ -963,8 +906,8 @@ class vmmManager(vmmGObjectUI):
         else:
             # Pop up connection menu
             conn = model[_iter][ROW_HANDLE]
-            disconn = (conn.get_state() == vmmConnection.STATE_DISCONNECTED)
-            conning = (conn.get_state() == vmmConnection.STATE_CONNECTING)
+            disconn = conn.is_disconnected()
+            conning = conn.is_connecting()
 
             self.connmenu_items["create"].set_sensitive(not disconn)
             self.connmenu_items["disconnect"].set_sensitive(not (disconn or
@@ -980,7 +923,9 @@ class vmmManager(vmmGObjectUI):
     #################
 
     def vmlist_name_sorter(self, model, iter1, iter2, ignore):
-        return cmp(model[iter1][ROW_SORT_KEY], model[iter2][ROW_SORT_KEY])
+        key1 = str(model[iter1][ROW_SORT_KEY]).lower()
+        key2 = str(model[iter2][ROW_SORT_KEY]).lower()
+        return cmp(key1, key2)
 
     def vmlist_guest_cpu_usage_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
