@@ -19,14 +19,12 @@
 
 import logging
 
-# pylint: disable=E0611
 from gi.repository import Gtk
 from gi.repository import GObject
-# pylint: enable=E0611
 
 import virtinst
-from virtManager import uiutil
-from virtManager.baseclass import vmmGObjectUI
+from . import uiutil
+from .baseclass import vmmGObjectUI
 
 
 class vmmNetworkList(vmmGObjectUI):
@@ -43,6 +41,7 @@ class vmmNetworkList(vmmGObjectUI):
         self.builder.connect_signals({
             "on_net_source_changed": self._on_net_source_changed,
             "on_net_source_mode_changed": self._emit_changed,
+            "on_net_portgroup_changed": self._emit_changed,
             "on_net_bridge_name_changed": self._emit_changed,
 
             "on_vport_type_changed": self._emit_vport_changed,
@@ -89,15 +88,20 @@ class vmmNetworkList(vmmGObjectUI):
         # [xml value, label]
         model = Gtk.ListStore(str, str)
         combo.set_model(model)
-        uiutil.set_combo_text_column(combo, 1)
+        uiutil.init_combo_text_column(combo, 1)
 
         model.append(["bridge", "Bridge"])
         model.append(["vepa", "VEPA"])
         model.append(["private", "Private"])
         model.append(["passthrough", "Passthrough"])
-
         combo.set_active(0)
 
+        combo = self.widget("net-portgroup")
+        # [xml value, label]
+        model = Gtk.ListStore(str, str)
+        combo.set_model(model)
+        uiutil.init_combo_text_column(combo, 1)
+
         self.conn.connect("net-added", self._repopulate_network_list)
         self.conn.connect("net-removed", self._repopulate_network_list)
         self.conn.connect("interface-added", self._repopulate_network_list)
@@ -124,35 +128,30 @@ class vmmNetworkList(vmmGObjectUI):
 
         return ret
 
-    def _build_source_row(self, nettype, name,
+    def _build_source_row(self, nettype, source_name,
         label, is_sensitive, is_running, manual_bridge=False, key=None):
-        return [nettype, name, label,
+        return [nettype, source_name, label,
                 is_sensitive, is_running, manual_bridge,
                 key]
 
     def _find_virtual_networks(self):
-        vnet_dict = {}
+        rows = []
         vnet_bridges = []
-        hasNet = False
-        netIdxLabel = None
+        default_label = None
 
-        for uuid in self.conn.list_net_uuids():
-            net = self.conn.get_net(uuid)
+        for net in self.conn.list_nets():
             nettype = virtinst.VirtualNetworkInterface.TYPE_VIRTUAL
 
             label = self._pretty_network_desc(nettype, net.get_name(), net)
             if not net.is_active():
                 label += " (%s)" % _("Inactive")
 
-            hasNet = True
-            # FIXME: Should we use 'default' even if it's inactive?
-            # FIXME: This preference should be configurable
             if net.get_name() == "default":
-                netIdxLabel = label
+                default_label = label
 
-            vnet_dict[label] = self._build_source_row(
+            rows.append(self._build_source_row(
                 nettype, net.get_name(), label, True,
-                net.is_active(), key=net.get_uuid())
+                net.is_active(), key=net.get_connkey()))
 
             # Build a list of vnet bridges, so we know not to list them
             # in the physical interface list
@@ -160,74 +159,81 @@ class vmmNetworkList(vmmGObjectUI):
             if vnet_bridge:
                 vnet_bridges.append(vnet_bridge)
 
-        if not hasNet:
+        if not rows:
             label = _("No virtual networks available")
-            vnet_dict[label] = self._build_source_row(
-                None, None, label, False, False)
+            rows.append(self._build_source_row(
+                None, None, label, False, False))
 
-        return vnet_dict, vnet_bridges, netIdxLabel
+        return rows, vnet_bridges, default_label
 
     def _find_physical_devices(self, vnet_bridges):
+        rows = []
+        can_default = False
+        default_label = None
+        skip_ifaces = ["lo"]
+
         vnet_taps = []
-        for vm in self.conn.vms.values():
+        for vm in self.conn.list_vms():
             for nic in vm.get_network_devices(refresh_if_nec=False):
                 if nic.target_dev and nic.target_dev not in vnet_taps:
                     vnet_taps.append(nic.target_dev)
 
-        bridge_dict = {}
-        iface_dict = {}
-        hasShared = False
-        brIdxLabel = None
-        skip_ifaces = ["lo"]
+        netdevs = {}
+        for iface in self.conn.list_interfaces():
+            netdevs[iface.get_name()] = [
+                iface.get_name(), iface.is_bridge(), iface.get_slave_names()]
+        for nodedev in self.conn.filter_nodedevs("net"):
+            if nodedev.xmlobj.interface not in netdevs:
+                netdevs[nodedev.xmlobj.interface] = [nodedev.xmlobj.interface,
+                    False, []]
 
-        for name in self.conn.list_net_device_paths():
-            br = self.conn.get_net_device(name)
-            bridge_name = br.get_bridge()
-            nettype = virtinst.VirtualNetworkInterface.TYPE_BRIDGE
+        # For every bridge used by a virtual network, and any slaves of
+        # those devices, don't list them.
+        for vnet_bridge in vnet_bridges:
+            slave_names = netdevs.pop(vnet_bridge, [None, None, []])[2]
+            for slave in slave_names:
+                netdevs.pop(slave, None)
 
-            if ((bridge_name in vnet_bridges) or
-                (br.get_name() in vnet_bridges) or
-                (br.get_name() in vnet_taps) or
-                (br.get_name() in [v + "-nic" for v in vnet_bridges]) or
-                (br.get_name() in skip_ifaces)):
+        for name, is_bridge, slave_names in netdevs.values():
+            if ((name in vnet_taps) or
+                (name in [v + "-nic" for v in vnet_bridges]) or
+                (name in skip_ifaces)):
                 # Don't list this, as it is basically duplicating
                 # virtual net info
                 continue
 
-            if br.is_shared():
-                sensitive = True
-                if br.get_bridge():
-                    hasShared = True
-                    brlabel = "(%s)" % self._pretty_network_desc(nettype,
-                                                                 bridge_name)
+            sensitive = True
+            source_name = name
+
+            label = _("Host device %s") % (name)
+            if is_bridge:
+                nettype = virtinst.VirtualNetworkInterface.TYPE_BRIDGE
+                if slave_names:
+                    extra = (_("Host device %s") % slave_names[0])
+                    can_default = True
                 else:
-                    bridge_name = name
-                    brlabel = _("(Empty bridge)")
-            else:
-                if self.conn.check_support(
+                    extra = _("Empty bridge")
+                label = _("Bridge %s: %s") % (name, extra)
+
+            elif self.conn.check_support(
                     self.conn.SUPPORT_CONN_DIRECT_INTERFACE):
-                    sensitive = True
-                    nettype = virtinst.VirtualNetworkInterface.TYPE_DIRECT
-                    bridge_name = name
-                    brlabel = ": %s" % _("macvtap")
-                else:
-                    sensitive = False
-                    brlabel = "(%s)" % _("Not bridged")
+                nettype = virtinst.VirtualNetworkInterface.TYPE_DIRECT
+                label += (": %s" % _("macvtap"))
 
-            label = _("Host device %s %s") % (br.get_name(), brlabel)
-            if hasShared and not brIdxLabel:
-                brIdxLabel = label
-
-            row = self._build_source_row(
-                nettype, bridge_name, label, sensitive, True,
-                key=br.get_name())
-
-            if sensitive:
-                bridge_dict[label] = row
             else:
-                iface_dict[label] = row
+                nettype = None
+                sensitive = False
+                source_name = None
+                label += (": %s" % _("Not bridged"))
 
-        return bridge_dict, iface_dict, brIdxLabel
+            if can_default and not default_label:
+                default_label = label
+
+            rows.append(self._build_source_row(
+                nettype, source_name, label, sensitive, True,
+                key=name))
+
+        return rows, default_label
 
     def _populate_network_list(self):
         net_list = self.widget("net-source")
@@ -243,22 +249,30 @@ class vmmNetworkList(vmmGObjectUI):
             net_list.set_active(0)
             return
 
-        (vnet_dict, vnet_bridges, netIdxLabel) = self._find_virtual_networks()
-        (bridge_dict, iface_dict, brIdxLabel) = self._find_physical_devices(
+        (vnets, vnet_bridges, default_net) = self._find_virtual_networks()
+        (iface_rows, default_bridge) = self._find_physical_devices(
             vnet_bridges)
 
-        for indict in [bridge_dict, vnet_dict, iface_dict]:
-            keylist = indict.keys()
-            keylist.sort()
-            rowlist = [indict[k] for k in keylist]
-            for row in rowlist:
+        # Sorting is:
+        # 1) Bridges
+        # 2) Virtual networks
+        # 3) direct/macvtap
+        # 4) Disabled list entries
+        # Each category sorted alphabetically
+        bridges = [row for row in iface_rows if row[0] == "bridge"]
+        direct = [row for row in iface_rows if row[0] == "direct"]
+        disabled = [row for row in iface_rows if row[0] is None]
+
+        for rows in [bridges, vnets, direct, disabled]:
+            rows.sort(key=lambda r: r[2])
+            for row in rows:
                 model.append(row)
 
         # If there is a bridge device, default to that
         # If not, use 'default' network
         # If not present, use first list entry
         # If list empty, use no network devices
-        label = brIdxLabel or netIdxLabel
+        label = default_bridge or default_net
 
         default = 0
         if not len(model):
@@ -284,15 +298,13 @@ class vmmNetworkList(vmmGObjectUI):
     ###############
 
     def get_network_row(self):
-        return uiutil.get_list_selection(self.widget("net-source"))
+        return uiutil.get_list_selection(self.widget("net-source"), None)
 
     def get_network_selection(self):
-        net_list = self.widget("net-source")
         bridge_entry = self.widget("net-bridge-name")
-
-        row = uiutil.get_list_selection(net_list)
+        row = self.get_network_row()
         if not row:
-            return None, None, None
+            return None, None, None, None
 
         net_type = row[0]
         net_src = row[1]
@@ -304,9 +316,13 @@ class vmmNetworkList(vmmGObjectUI):
 
         mode = None
         if self.widget("net-source-mode").is_visible():
-            mode = uiutil.get_list_selection(self.widget("net-source-mode"), 0)
+            mode = uiutil.get_combo_entry(self.widget("net-source-mode"), 0)
 
-        return net_type, net_src, mode
+        portgroup = None
+        if self.widget("net-portgroup").is_visible():
+            portgroup = uiutil.get_combo_entry(self.widget("net-portgroup"), 0)
+
+        return net_type, net_src, mode, portgroup or None
 
     def get_vport(self):
         vport_type = self.widget("vport-type").get_text()
@@ -319,7 +335,7 @@ class vmmNetworkList(vmmGObjectUI):
          vport_idver, vport_instid)
 
     def validate_network(self, macaddr, model=None):
-        nettype, devname, mode = self.get_network_selection()
+        nettype, devname, mode, portgroup = self.get_network_selection()
         if nettype is None:
             return None
 
@@ -328,7 +344,7 @@ class vmmNetworkList(vmmGObjectUI):
         # Make sure VirtualNetwork is running
         netobj = None
         if nettype == virtinst.VirtualNetworkInterface.TYPE_VIRTUAL:
-            for net in self.conn.nets.values():
+            for net in self.conn.list_nets():
                 if net.get_name() == devname:
                     netobj = net
                     break
@@ -344,7 +360,6 @@ class vmmNetworkList(vmmGObjectUI):
             # Try to start the network
             try:
                 netobj.start()
-                netobj.tick()
                 logging.info("Started network '%s'", devname)
             except Exception, e:
                 return self.err.show_err(_("Could not start virtual network "
@@ -358,6 +373,7 @@ class vmmNetworkList(vmmGObjectUI):
             net.macaddr = macaddr
             net.model = model
             net.source_mode = mode
+            net.portgroup = portgroup
             if net.model == "spapr-vlan":
                 net.address.set_addrstr("spapr-vio")
 
@@ -388,13 +404,19 @@ class vmmNetworkList(vmmGObjectUI):
     def reset_state(self):
         self._populate_network_list()
 
+        net_err = None
+        if (not self.conn.is_nodedev_capable() or
+            not self.conn.is_interface_capable()):
+            net_err = _("Libvirt version does not support "
+                        "physical interface listing.")
+
         net_warn = self.widget("net-source-warn")
-        net_err = self.conn.netdev_error
         net_warn.set_visible(bool(net_err))
         net_warn.set_tooltip_text(net_err or "")
 
         self.widget("net-bridge-name").set_text("")
         self.widget("net-source-mode").set_active(0)
+        self.widget("net-portgroup").get_child().set_text("")
 
         self.widget("vport-type").set_text("")
         self.widget("vport-managerid").set_text("")
@@ -443,6 +465,9 @@ class vmmNetworkList(vmmGObjectUI):
         combo.set_active_iter(rowiter)
         combo.emit("changed")
 
+        if net.portgroup:
+            uiutil.set_combo_entry(self.widget("net-portgroup"), net.portgroup)
+
 
     #############
     # Listeners #
@@ -471,15 +496,27 @@ class vmmNetworkList(vmmGObjectUI):
                 netlist.set_active_iter(row.iter)
                 return
 
-    def _on_net_source_changed(self, src):
-        self._emit_changed()
+    def _populate_portgroups(self, portgroups):
+        combo = self.widget("net-portgroup")
+        model = combo.get_model()
+        model.clear()
 
-        row = uiutil.get_list_selection(src)
+        default = None
+        for p in portgroups:
+            model.append([p.name, p.name])
+            if p.default:
+                default = p.name
+
+        uiutil.set_combo_entry(combo, default)
+
+    def _on_net_source_changed(self, src):
+        ignore = src
+        self._emit_changed()
+        row = self.get_network_row()
         if not row:
             return
 
         is_direct = (row[0] == virtinst.VirtualNetworkInterface.TYPE_DIRECT)
-
         self.widget("vport-expander").set_visible(is_direct)
         uiutil.set_grid_row_visible(self.widget("net-source-mode"), is_direct)
         uiutil.set_grid_row_visible(
@@ -490,3 +527,12 @@ class vmmNetworkList(vmmGObjectUI):
         show_bridge = row[5]
         uiutil.set_grid_row_visible(
             self.widget("net-bridge-name"), show_bridge)
+
+        portgroups = []
+        connkey = row[6]
+        if connkey and row[0] == virtinst.VirtualNetworkInterface.TYPE_VIRTUAL:
+            portgroups = self.conn.get_net(connkey).get_xmlobj().portgroups
+
+        uiutil.set_grid_row_visible(
+            self.widget("net-portgroup"), bool(portgroups))
+        self._populate_portgroups(portgroups)
